/*
 * Cardinality.java
 *
 * Created on 12. oktober 2005, 12:46
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import jcreek.representation.constraints.ConstraintCheckResult;
/**
 * Handles the behaviour of Cardinality constraints. <p>
 * Offers the ability from define cardinality constraints for the relations
 * of a given type for some entity. Two integer values, max and min defines
 * the maximum and minimum cardinality.
 *
 * @author  Martin Stige
 */
public class CardinalityConstraintObject extends ConstraintObject{
    private int numberMin;
    private int numberMax;
    
    /** Creates a new instance of Cardinality
     * <p>
     * The entity <code>from</code> must have minimum <code>numberMin</code>
     * and maximum <code>numberMax</code> values for the slot/relation <code>
     * rel</code>
     *
     * @param rel the RelationType being constraind
     * @param numberMin the minimum number of values
     * @param numberMax the maximum number of values
     */
    public CardinalityConstraintObject(RelationType rel, int numberMin, int numberMax) {
        this.numberMin = numberMin;
        this.numberMax = numberMax;
        this.rel = rel.getName();
        
    }
    
    public ConstraintCheckResult check(Entity from) {
        Relation[] rels = from.getRelations(getRelation());
        String result = "";
        boolean ok = true;
        
        if(rels.length < numberMin)
            ok = false;
        else if(rels.length > numberMax && numberMax != -1)
            ok = false;
        
        if(!ok){
            if(numberMin == -1)
                result += "Cardinality for " + from.getName() + " -> " + getRelation().getName() + " should be less than, or equal to " + numberMax +"\n";
            else if(numberMax == -1)
                result += "Cardinality for " + from.getName() + " -> " + getRelation().getName() + " should be greater than, or equal to " + numberMin + "\n";
            else
                result += "Cardinality for " + from.getName() + " -> " + getRelation().getName() + " should be " + numberMin + " - " + numberMax +"\n";
            for(int i = 0; i < rels.length; i++){
                Relation r = rels[i];
                result += from.getName() + " -> " + rels[i].getName() + " -> " + rels[i].getValue().getName() + "\n";
            }
        }
        return new ConstraintCheckResult(ok, result);
    }
    
    public String toString(){
        String ret = "Cardinality(";
        //ret += from.getName() + ", ";
        ret += getRelation().getName() + ", ";
        ret += (numberMin == -1? "-inf": numberMin+"") + ", ";
        ret += (numberMax == -1? "+inf": numberMax+"");
        ret += ")";
        
        return ret;
    }
    
    /** Returns the max cardinality
     * @return the max cardinality value
     */
    protected int getMax(){ return numberMax; }
    
    /** Returns the min cardinality
     * @return the max cardinality value
     */
    protected int getMin(){ return numberMin; }
    
    
    /** Sets the max cardinality
     * @param m the max cardinality value
     */
    protected void setMax(int m){ numberMax = m; }
    
    
    /** Sets the min cardinality
     * @param m the min cardinality value
     */
    protected void setMin(int m){ numberMin = m; }
    
    /** Creates a Cardinality constraint object
     * @param km the knowledge model
     * @param data the constraint object to be contained by the constraint
     */
    public Constraint newConstraintInstance(KnowledgeModel km, ConstraintData data){
        return new CardinalityConstraint(km, data);
    }
    
}
