/*
 * ConcreteValueConstraint.java
 *
 * Created on 13. oktober 2005, 09:40
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import jcreek.representation.cbr.*;
import jcreek.representation.constraints.ConstraintCheckResult;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Handles the behaviour of Concrete value constraints. <p>
 * Offers the ability to define constraints based on concrete values.
 * You can either define the closed set of allowed values, or define
 * a set of values that are not allowed.
 * <p>
 * It is not interresting to define both these two constraints on the same
 * entity->relation. If the (positive) allowed is used, all values not
 * on the list are implicitly defined as disallowed. <br>
 * If the (negative) disallowed is used all values not on the list are allowed.
 *
 * @author  Martin Stige
 */
public class ConcreteValueConstraintObject extends ConstraintObject{
    private ArrayList values;
    private boolean allowed;
    
    /** Creates a new instance of ConcreteValueConstraintObject
     *
     * Defines a list of values/entities that are either allowed or dissallowed
     * as a value for a relation on an entity.
     * <p>
     * Meaning of constraint: The values in the list are allowed/dissallowed
     * as the values of relations of type rel from entity from
     *
     * @param rel the relation type being restricted
     * @param values the values allowed or dissallowed
     * @param allowed descides whether the values are allowed or dissallowed
     *
     */
    public ConcreteValueConstraintObject(RelationType rel, ArrayList values, boolean allowed) {
        this.rel = rel.getName();
        setValues(values);
        this.allowed = allowed;
    }
    
    /** Adds a new value to the allowed/dissallowed list
     *
     * @param val the value to be added
     */
    protected void addValue(Entity val){ values.add(val.getID()); }
    
    /**
     * Checks the constraint
     * @param from the entity that are constrained
     */
    public ConstraintCheckResult check(Entity from) {
        Relation[] rels = from.getRelations(getRelation());
        String[] errors = new String[rels.length];
        int errorIndex = 0;
        
        for(int i = 0; i < rels.length; i ++){
            Relation r = rels[i];
            Entity e = r.getValue();
            
            Iterator it = new MyEntityIterator(values.iterator());
            boolean ok = false;
            
            //allowed
            if(allowed){
                while(it.hasNext() && !ok){
                    Entity listVal = (Entity)it.next();
                    
                    if(e.equals(listVal))
                        ok = true;
                }
            }else{
                ok = true;
                while(it.hasNext() && ok){
                    Entity listVal = (Entity)it.next();
                    
                    if(e.equals(listVal))
                        ok = false;
                }
            }
            if(!ok)
                errors[errorIndex++] = from.getName() + " -> " + r.getName() + " -> " + e.getName();
        }
        
        String result = "";
        if(errorIndex > 0){
            if(allowed){
                result += "Concrete values allowed: ";
                
                ;
                for(Iterator it = new MyEntityIterator(values.iterator()); it.hasNext(); )
                    result += ((Entity)it.next()).getName() + ", ";
                result += "\n";
            }
            
            for(int i = 0; i < errorIndex; i++)
                result += "  " + errors[i] +"\n";
            return new ConstraintCheckResult(false, result);
        }else
            return new ConstraintCheckResult(true, result);
        
    }
    
    /**
     * Returns the values that are allowed or disallowed
     */
    protected ArrayList getValues() {
        ArrayList result = new ArrayList(values.size());
        
        for(Iterator it = new MyEntityIterator(values.iterator()); it.hasNext(); )
            result.add(it.next());
        
        return result;
    }
    
    /**
     * Returns whether the values are allowed or not<br>
     * true = allowed, false = disallowed
     */
    protected boolean getAllowed() { return allowed; }
    
    /**
     * Sets the list of values
     * @param a the list of values
     */
    protected void setValues(ArrayList a){
        values = new ArrayList(a.size());
        
        for(Iterator it = a.iterator(); it.hasNext(); )
            values.add(((Entity)it.next()).getID());
        
        System.out.println("Values addded " + values.size());
    }
    
    /**
     * Sets wheter the list of values are allowed or not
     */
    protected void setAllowed(boolean b){ allowed = b; }
    
    /**
     * Creates a ConcretValueConstraint object from a ConstraintData object
     * @param km the knowledge model
     * @param data the constraint data to be contained by the constraint
     */
    public Constraint newConstraintInstance(KnowledgeModel km, ConstraintData data) {
        return new ConcreteValueConstraint(km, data);
    }
    
    public String toString(){
        String ret = "Concrete value(";
        // ret += from.getName() + ", ";
        ret += getRelation().getName() + ", ";
        ret += (allowed ? "allowed" : "disallowed");
        ret += ", {";
        
        for(Iterator it = new MyEntityIterator(values.iterator()); it.hasNext(); )
            ret += ((Entity)it.next()).getName() + ", ";
        ret += "})";
        
        return ret;
    }
    
    /**
     * Iterator for iterating an ArrayList containing Entity-objects
     */
    private class MyEntityIterator implements Iterator {
        Iterator superIterator;
        
        MyEntityIterator(Iterator it) { superIterator = it; }
        
        public Object next() { return km.getEntity(superIterator.next()); }
        
        public boolean hasNext() { return superIterator.hasNext(); }
        
        public void remove() { superIterator.remove(); }
    }
}
