/*
 * Constraint.java
 *
 * Created on 5. januar 2006, 14:14
 */

package jcreek.representation.constraints;


import jcreek.representation.*;
import jcreek.representation.constraints.ConstraintCheckResult;
/**
 * Super class for all the different Constraint classes
 * @author  Martin Stige
 */
public abstract class Constraint {
    protected transient KnowledgeModel knowledgeModel;
    protected transient ConstraintData constraintData;
    
    /**
     * Creates a new instance of Constraint
     * @param knowledgeModel the knowledge model
     * @param description a description of the constraint
     */
    public Constraint(KnowledgeModel knowledgeModel, String description) {
        this.knowledgeModel = knowledgeModel;
        this.constraintData = knowledgeModel.makeConstraint();
        
        setDescription(description);
        
        //Not implemented
        //constraintData.setCreationTime(System.currentTimeMillis());
        //knowledgeModel.fireModelChangeEvent(new ModelChangeEvent(this, ModelChangeEvent.ENTITY_ADDED));
    }
    
    /**
     * Creates a new instance of CardinalityConstraint from a constraint data object
     * @param knowledgeModel the knowledge model
     * @param constraintData the constraint data to be contained in the constraint
     */
    public Constraint(KnowledgeModel knowledgeModel, ConstraintData constraintData){
        this.knowledgeModel = knowledgeModel;
        this.constraintData = constraintData;
    }
    
    /**
     * Checks the constraint, met or violated
     * @param e the entity to be checked with this constraint
     */
    public ConstraintCheckResult check(Entity e){
        return constraintData.getConstraintObject().check(e);
    }
    
    /**
     * Sets the description of this constraint.
     * @param description the description of the constraint
     */
    public void setDescription(String description) {
        constraintData.setDescription(description);
    }
    
    /** Returns the description of the constraint */
    public String getDescription(){ return constraintData.getDescription(); }
    
    /**
     * Sets the relation type constrained by the constraint
     * @param rel the relation
     */
    public void setRelation(RelationType rel) { constraintData.getConstraintObject().setRelation(rel); }
    
    /**
     * Returns the relation type
     */
    public RelationType getRelation(){ return constraintData.getConstraintObject().getRelation(); }
    
    /** Returns the constraint data */
    public ConstraintData getConstraintData(){ return constraintData; }
    
    /**
     * Sets the knowledge model that this constraint contains
     * @param knowledgeModel the knowledge model
     */
    public void setKnowledgeModel(KnowledgeModel knowledgeModel) {this.knowledgeModel = knowledgeModel; }
    
    /**
     * Descides whether two constraints are equal
     * @param obj the constraint to be compared with this
     */
    public boolean equals(Object obj){
        try{
            return ((Constraint)obj).getID() == getID();
        }catch(Exception e){
            // If an error is thrown, we assume the objects are not equal
            return false;
        }
    }
    
    /** Returns the ID of the constraint */
    public Integer getID(){ return constraintData.getID(); }
    
    /** Returns a string describing the constraint */
    public String toString(){ return constraintData.toString(); }
    
    
}
