/*
 * DJavaAdapter.java
 *
 * Created on 13. februar 2006, 11:44
 */

package jcreek.representation.constraints;

import java.io.*;

import koala.dynamicjava.interpreter.*;
import koala.dynamicjava.parser.wrapper.*;

import jcreek.representation.Entity;

/**
 * Adapter used as an interface to the DynamicJava source interpreter
 *
 * DynamicJava is distributed under the following license: <p>
 *
 * <i>DynamicJava - Copyright  1999 Dyade <p>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * <p>
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * <p>
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DYADE BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 * <i>
 * <p>
 * @author  Martin Stige
 */
public class DJavaAdapter {
    Entity entity;
    TreeInterpreter interpreter;
    String  script =
    "import jcreek.representation.*; \n" +
    "import jcreek.representation.Relation; \n" +
    "import jcreek.representation.Entity; \n" +
    "import jcreek.representation.RelationType; \n " +
    "import jcreek.representation.constraints.ConstraintCheckResult;\n\n";
    
    
    /**
     * Creates a new instance of DJavaAdapter
     * @param entity the entity that are subject to constraint checking
     */
    public DJavaAdapter(Entity entity) {
        interpreter = new TreeInterpreter(new JavaCCParserFactory());
        
        this.entity = entity;
    }
    
    /**
     * Performes a constraint check. The constraint is described as java
     * source code in a text string
     *
     * @param scr the source code
     */
    public ConstraintCheckResult validate(String scr){
        script += scr;
        script += "\n check(entity);";
        
        System.out.println("Validating: \n\n"+script);
        
        ConstraintCheckResult result = new ConstraintCheckResult();
        interpreter.defineVariable("entity", entity);
        
        try{
            Object res = validate();
            if(res == null)
                return new ConstraintCheckResult(false, "Evaluation of general constraint timed out");
            
            return (ConstraintCheckResult) res;
        }catch(IOException e){
            e.printStackTrace();
        }catch(InterpreterException e){
            return new ConstraintCheckResult(false, "Evaluation of general constraint failed: " + e.getMessage());
        }catch(ClassCastException e){
            e.printStackTrace();
        }
        return null;
        
    }
    
    /** Performes a syntax check on the source code in the string
     * @param scr the source code
     */
    public String checkSyntax(String scr){
        /* This should be reimplemented because it is using the running instance
         * of the knowledge model and the entity. Errors in the code can change
         * the knowledge model
         */
        script += scr;
        script += "\n check(entity);";
        
        ConstraintCheckResult result = new ConstraintCheckResult();
        interpreter.defineVariable("entity", entity);
        
        try{
            Object res = validate(); //interpreter.interpret(makeStream(), "test");
            if(res == null)
                return "evaluation timed out";
            return "syntax ok \n" + res.toString();
            
        }catch(IOException e){
            e.printStackTrace();
        }catch(InterpreterException e){
            return "Errors interpreting code: \n\n" + e.getMessage() + "\n\n" + scriptWidthLine();
        }catch(ClassCastException e){
            e.printStackTrace();
        }
        return null;
        
    }
    
    public String testSyntax(String scr){
        /* This should be reimplemented because it is using the running instance
         * of the knowledge model and the entity. Errors in the code can change
         * the knowledge model
         */
        script += scr;
        System.out.println(script);
        
        try{
            Object res = validate(); //interpreter.interpret(makeStream(), "test");
            if(res == null)
                return "evaluation timed out";
            return "syntax ok \n" + res.toString();
            
        }catch(IOException e){
            e.printStackTrace();
        }catch(InterpreterException e){
            return "Errors interpreting code: \n\n" + e.getMessage() + "\n\n" + scriptWidthLine();
        }catch(ClassCastException e){
            e.printStackTrace();
        }
        return null;
        
    }
    
    /** Runs the Dynamic Java interpreter on the script-string. This is done in
     * a separate thread to allow the system to time out if the interpreter
     * uses loooong time.
     */
    private Object validate() throws IOException, InterpreterException{
        Validator val = new Validator();
        val.start();
        
        long start = System.currentTimeMillis();
        System.out.println("Starting thread \n" + start);
        
        while(!val.isFinished() && ((System.currentTimeMillis() - start) < 20000)){
            //System.out.println(System.currentTimeMillis() + "\t " + sekunder(System.currentTimeMillis() - start));
        }
        System.out.println("\nFinished " + sekunder(System.currentTimeMillis() - start));
        return val.getResult();
    }
    
    /** Makes a PipedReader stream of the script */
    private PipedReader makeStream() throws IOException {
        PipedWriter pw = new PipedWriter();
        PipedReader pr = new PipedReader(pw);
        pw.write(script);
        
        pw.flush();
        pw.close();
        
        return pr;
    }
    
    private String makeFile() throws IOException{
        String file = "c:\\constraint.txt";
        BufferedWriter bw = new BufferedWriter(new FileWriter(file));
        
        bw.write(script);
        bw.flush();
        bw.close();
        return file;
    }
    
    /**
     * Add to the end of the script string
     * @param str the string to be added
     */
    public void add(String str){ script += str; }
    
    /** Returns the script formated with line numbers to ease debugging */
    private String scriptWidthLine(){
        String newScript = "1 ";
        int line = 2;
        for(int i = 0; i < script.length(); i++){
            if(script.substring(i, i+1).equals("\n"))
                newScript += "\n" + (line++) + " ";
            else
                newScript += script.substring(i, i+1);
        }
        
        return newScript;
    }
    
    private String sekunder(long t){
        String tall = ""+ t;
        String ut = "";
        
        if(t > 99)
            ut = tall.substring(tall.length()-2, tall.length()-1) + ut;
        if(t > 999)
            ut = tall.substring(tall.length()-3, tall.length()-2) + ut;
        ut = "," + ut;
        
        for(int i = tall.length()-3; i > 0; i--)
            ut = tall.substring(i-1, i) + ut;
        
        return ut;
    }
    
    /** A simple class that runs the Dynamic Java interpreter as
     * a separate thread
     */
    private class Validator extends Thread {
        Object res = null;
        boolean finished = false;
        IOException ioe = null;
        InterpreterException iee = null;
        
        public void run() {
            try{
                /*
                 * The DynamicJava system is made to be able to get the source
                 * code to validate in different ways. Files, streams etc. I
                 * tried to do it using streams, but this did not work in all
                 * cases. I therefore use a file to send the constraint. This is
                 * probably not the preffered way to do it, but it works.
                 */
                //res = interpreter.interpret(makeStream(), "test");
                res = interpreter.interpret(makeFile());
                finished = true;
            }catch(IOException e){
                ioe = e;
                finished = true;
            }catch(InterpreterException e){
                iee = e;
                finished = true;
            }
        }
        
        boolean isFinished(){ return finished; }
        
        Object getResult() throws InterpreterException, IOException{
            
            if(ioe != null)
                throw ioe;
            if(iee != null)
                throw iee;
            return res;
        }
        
    }
}

