/*
 * ValueRangeConstraint.java
 *
 * Created on 9. november 2005, 15:15
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import jcreek.representation.cbr.*;
import jcreek.representation.constraints.ConstraintCheckResult;

/**
 * Handles the behaviour of Value range constraints<p>.
 * Offers the ability to constrain the value of nummerical slots.
 * You can either define an allowed range of values or a range that is not allowed
 *
 * @author  Martin Stige
 */
public class ValueRangeConstraintObject extends ConstraintObject {
    private Number valueMin;
    private Number valueMax;
    
    private boolean allowed;
    
    /** Creates a new instance of ValueRangeConstraint
     * <p>
     * Constrains the values of relations pointing to nummerical values. You
     * can either define an allowed or disallowed range of values.
     *
     * @param rel the RelationType being constraind
     * @param valueMin the minimum number of values
     * @param valueMax the maximum number of values
     * @param allowed whether the range (valueMax - valueMin) is allowed or disallowed
     *
     */
    public ValueRangeConstraintObject(RelationType rel, Number valueMin, Number valueMax, boolean allowed) {
        this.valueMin = valueMin;
        this.valueMax = valueMax;
        
        this.rel = rel.getName();
        
        this.allowed = allowed;
    }
    
    public ConstraintCheckResult check(Entity from) {
        Relation[] rels = from.getRelations(getRelation());
        boolean ok = true;
        String result = "";
        
        for(int i = 0; i < rels.length; i++){
            Entity e = rels[i].getValue();
            
            if(NumberEntity.matches(e)){
                //Number n = ((NumberEntity)e).asNumber(); This should have worked, but the entity is not necessarily of type NumberEntity
                Number n = (Number) e.getEntityObject();
                
                if(valueMin == null){
                    if((n.floatValue() > valueMax.floatValue()) && allowed){
                        result += from.getName() + " -> " + getRelation().getName() + " -> " + n.floatValue();
                        result += "  |  Allowed (n < " + valueMax.floatValue() + ") \n";
                        ok = false;
                    }else if(!(n.floatValue() < valueMax.floatValue()) && !allowed){
                        result += "Violation: " + from.getName() + " -> " + getRelation().getName() + " -> " + n.floatValue();
                        result += "  |  Disallowed (n < " + valueMax.floatValue() + ") \n";
                        ok = false;
                        
                    }
                }else if(valueMax == null){
                    if((n.floatValue() < valueMin.floatValue()) && allowed){
                        result += from.getName() + " -> " + getRelation().getName() + " -> " + n.floatValue();
                        result += "  |  Allowed (" + valueMin.floatValue() + " < n) \n";
                        ok = false;
                    }else if(!(n.floatValue() > valueMin.floatValue()) && !allowed){
                        result += "Violation: " + from.getName() + " -> " + getRelation().getName() + " -> " + n.floatValue();
                        result += "  |  Disallowed (" + valueMin.floatValue() + " < n) \n";
                        ok = false;
                    }
                    
                }else{
                    
                    if((n.floatValue() < valueMin.floatValue() || n.floatValue() > valueMax.floatValue()) && allowed){
                        result += from.getName() + " -> " + getRelation().getName() + " -> " + n.floatValue();
                        result += "  |  Allowed (" + valueMin.floatValue() + " - " + valueMax.floatValue() + ") \n";
                        ok = false;
                    }else if(!(n.floatValue() > valueMin.floatValue() && n.floatValue() < valueMax.floatValue()) && !allowed){
                        result += "Violation: " + from.getName() + " -> " + getRelation().getName() + " -> " + n.floatValue();
                        result += "  |  Disallowed (" + valueMin.floatValue() + " - " + valueMax.floatValue() + ") \n";
                        ok = false;
                    }
                }
            }
        }
        
        
        return new ConstraintCheckResult(ok, result);
    }
    
    public Constraint newConstraintInstance(KnowledgeModel km, ConstraintData data) {
        return new ValueRangeConstraint(km, data);
    }
    
    protected Number getMax(){ return valueMax; }
    protected Number getMin(){ return valueMin; }
    
    protected boolean getAllowed() { return allowed; }
    
    protected void setMax(Number n){ valueMax = n; }
    protected void setMin(Number n){ valueMin = n; }
    
    protected void setAllowed(boolean b){ allowed = b; }
    
    public String toString(){
        String ret = "Value range(";
        //ret += from.getName() + ", ";
        ret += getRelation().getName() + ", ";
        ret += (valueMin == null? "-inf" : valueMin+"") + ", ";
        ret += (valueMax == null? "+inf" : valueMax+"") + ", ";
        ret += (allowed ? "allowed" : "disallowed");
        ret += ")";
        
        return ret;
    }
    
}
